import java.io.*;
import java.util.*;


/**
 * Trida pro nacteni topologie site, parametru site a trenovaci mnoziny ze
 * souboru.
 * @author Martin Kot
 * @version 1.0 2001-05-15
 */
public class FileParser {
  /*Jmeno vstupniho souboru*/
  private String fileName = null;
  /*Pocty neuronu ve vrstvach od nejnizsi*/
  private int[] neuronsCounts;
  /*Vector poli vstupnich hodnot trenovaci mnoziny*/
  private Vector trainIn; 
  /*Vector poli vystupnich hodnot trenovaci mnoziny*/
  private Vector trainOut;
  /*Urcuje, zda jiz soubor byl parsovan*/
  private boolean parsed; 

  
  /**
  * Konstruktor
  */
  public FileParser() {
    neuronsCounts = null;  
    trainIn = new Vector();
    trainOut = new Vector();
    parsed = false;
  }
  
  /**
  *  Nastavi si hodnotu, zda jiz byl soubor parsovan.
  */
  public void setParsed(boolean b) {
    parsed = b;
  }
  
 /**
  * Pokud jeste soubor neni zparsovan, provede parsovani. Slouzi k ziskani 
  * poctu neuronu v jednotlivych vrstvach
  * @return Pole poctu neuronu v jednotlivych vrstvach, pocinaje vstupni 
  * vrstvou. Neexistuji-li minimalne dve vrstvy vraci null.
  */
  public int[] getNeuronsCounts() {
    if (!parsed) {
      parse();
    }
    if (neuronsCounts == null){
      return null;
    }
    if (neuronsCounts.length < 2) {
      return null;
    }
    return neuronsCounts;
  }

 /**
  *  Z datoveho souboru nacte vstupni hodnoty trenovaci mnoziny.
  * @return Vektor poli vstupnich hodnot trenovaci mnoziny.
  */
  public Vector getTrainIns() {
    if (!parsed) {
      parse();
    }
    return trainIn;
  }

 /**
  * Z datoveho souboru nacte vystupni hodnoty trenovaci mnoziny.
  * @return Vektor poli vystupnich hodnot trenovaci mnoziny.
  */
  public Vector getTrainOuts() {
    if (!parsed) {
      parse();
    }
    return trainOut;
  }

 /**
  * Zadani jmena vstupniho souboru
  * @param fileName jmeno souboru
  */
  public void setFileName(String fileName) {
    if(fileName!=null){
        parsed = false;
        this.fileName = fileName;
    }
  }
  
  /**
  *  Zjisteni nazvu souboru s topologii site.
  *  @return jmeno souboru
  */
  public String getFileName() {
    return fileName;
  }

  /**
   * Provede parsovani souboru. Pokud je v souboru chyba, nastavi pole poctu 
   * neuronu na null.
   *
   */
  protected void parse(){
    if(!parsing()){
      neuronsCounts = null;  
    }
  }
       
  /**
  * Realizuje samotne parsovani souboru  
  * @return true, pokud ma soubor spravnou syntaxi, false pri spatne syntaxi
  */
  protected boolean parsing() {
    String line;
    Double value;
    Integer intvalue;
    BufferedReader in;
    
    int lineCounter = 0;
    
    try {
      in = new BufferedReader(new FileReader(fileName));
      trainIn.removeAllElements();
      trainOut.removeAllElements();
            
      parsed = true;
      while ((line = in.readLine()) != null) {
        lineCounter++;
        
        if (line.trim().startsWith("#") || line.trim().length() == 0) 
          continue;        
      
        if (line.trim().startsWith("Vrstva:")) {
          if((neuronsCounts = parseLayer(line))==null){
            return false;
          }
          continue;
        }  
        if (line.trim().startsWith("Strmost:")) {
          if ((value = parseLine(line)) != null) {
            NeuralNet.slope = value.doubleValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Min. vaha:")) {
          if ((value = parseLine(line)) != null) {
            NeuralNet.minWeight = value.doubleValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Max. vaha:")) {
          if ((value = parseLine(line)) != null) {
            NeuralNet.maxWeight = value.doubleValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Mi:")) {
          if ((value = parseLine(line)) != null) {
            NeuralNet.mi = value.doubleValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Ny:")) {
          if ((value = parseLine(line)) != null) {
            NeuralNet.ny = value.doubleValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Max. chyba:")) {
          if ((value = parseLine(line)) != null) {
            NeuralNet.maxError= value.doubleValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Opakovat polozku:")) {
          if ((intvalue = parseIntLine(line)) != null) {
            NeuralNet.repItem = intvalue.intValue();
            continue;
          }
        }  
        if (line.trim().startsWith("Opakovat mnozinu:")) {
          if ((intvalue = parseIntLine(line)) != null) {
            NeuralNet.repSet = intvalue.intValue();
            continue;
          }
        }  

        if (parseTrain(line)) {
          continue;
        }
        System.err.println(fileName+":"+lineCounter+
		" Chyba syntaxe vstupniho souboru");
        in.close();
        return false;
      }
      in.close();
    }
    catch(IOException ex) {
      System.err.println("Nepodarilo se otevrit vstupni soubor: "+fileName);
      return false;
    }
    return true;
  }
 
  /**
   * Slouzi k analyze radku, na kterem je topologie site
   * @param line Analyzovany radek souboru
   * @return Pocty neuronu v jednotlivych vrstvach
   */
  protected int[] parseLayer(String line){
    StringTokenizer t;
    int v[];
    int prvku;

    t = new StringTokenizer(line.substring(line.indexOf(":") + 1), ",");
    prvku=t.countTokens();
    
    v = new int[prvku];
    try {
      for(int i=0;i<prvku;i++){
        v[i]= new Integer(t.nextToken().trim()).intValue();
      }
    }
    catch(NumberFormatException e) {
      return null;
    }
    return v;
  }
 
  /**
   * Slouzi k analyze radku, na kterem je nejaka realna hodnota konstanty
   * @param line Analyzovany radek souboru
   * @return Nactena realna hodnota
   */
  protected Double parseLine(String line) {
    int strednik;
    if ((strednik = line.indexOf(":") + 1) < 1) {
      return null;
    }
    try {
      return new Double(line.substring(strednik).trim());
    }
    catch(NumberFormatException e) {
      return null;
    }
  }
  
  /**
   * Slouzi k analyze radku, na kterem je nejaka celociselna hodnota konstanty
   * @param line Analyzovany radek souboru
   * @return Nactena celociselna hodnota
   */
  protected Integer parseIntLine(String line) {
    int strednik;
    if ((strednik = line.indexOf(":") + 1) < 1) {
      return null;
    }
    try {
      return new Integer(line.substring(strednik).trim());
    }
    catch(NumberFormatException e) {
      return null;
    }
  }

  /**
   * Slouzi k analyze radku, na kterem je prvek trenovaci mnoziny. Jde o n-tici
   * vstupnich hodnot a n-tici vystupnich hodnot jednoho prvku.
   * @param line Analyzovany radek souboru
   * @return true, pokud ma radek spravnou syntaxi
   *	     false, pokud ma radek spatnou syntaxi
   */
  protected boolean parseTrain(String line) {
    StringTokenizer t;
    double v[];
    int prvku;
    int vetsitko;
    
    if ((vetsitko=line.indexOf(">"))==-1){
      return false;
    }
    //Vstupni data trenovaci mnoziny
    t = new StringTokenizer(line.substring(1, vetsitko), ",");
    if(neuronsCounts==null) {
      return false;
    }
    prvku=neuronsCounts[0];

    if (t.countTokens()!=prvku) {
      return false;
    }
    v = new double[prvku];
    try {
      for(int i=0;i<prvku;i++){
        v[i]= new Double(t.nextToken().trim()).doubleValue();
      }
    }
    catch(NumberFormatException e) {
      return false;
    }
    trainIn.add(v); 

    //Vystupni data trenovaci mnoziny
    t = new StringTokenizer(line.substring(line.indexOf(">") + 1), ",");
    prvku=neuronsCounts[neuronsCounts.length - 1];
    if  (t.countTokens() != prvku) {
      return false;
    }
    v = new double[prvku];
    try {
      for(int i=0;i<prvku;i++){
        v[i]= new Double(t.nextToken().trim()).doubleValue();
      }
    }
    catch(NumberFormatException e) {
      return false;
    }
    trainOut.add(v);
    return true;
  }
}
